==> 0.5.1.tar.gz: lib/src/sections/SectionTile.dart <==
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

import '../../dotup_flutter_widgets.dart';

class SectionTile extends StatelessWidget {
  final String? text;
  final Widget? icon;
  final VoidCallback? onTap;
  final Color? color;
  final Widget? subTitle;
  final Widget? title;
  final Widget? trailing;
  final bool dense;

  const SectionTile({
    Key? key,
    this.text,
    this.icon,
    this.onTap,
    this.color,
    this.title,
    this.subTitle,
    this.trailing,
    this.dense = false,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ListTile(
      contentPadding: EdgeInsets.symmetric(horizontal: 10),
      dense: dense,
      minLeadingWidth: 10,
      leading: icon,
      title: title ??
          Text(
            text!,
            style: TextStyle(color: color),
          ),
      subtitle: subTitle,
      trailing: trailing,
      onTap: onTap,
      enabled: onTap != null,
    );
  }

  factory SectionTile.navigator({
    Key? key,
    required String text,
    Icon? icon,
    required VoidCallback onTap,
    required Color color,
    String? subTitle,
    bool dense = false,
  }) {
    return SectionTile(
      text: text,
      color: color,
      dense: dense,
      trailing: Icon(
        Icons.keyboard_arrow_right,
        color: color,
      ),
      onTap: onTap,
    );
  }

  factory SectionTile.button({
    Key? key,
    required String text,
    Icon? icon,
    required VoidCallback onTap,
    required Color color,
    String? subTitle,
    bool dense = false,
  }) {
    final textWidget = Text(
      text,
      style: TextStyle(color: color),
    );

    final title = icon == null
        ? textWidget
        : Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Padding(
                padding: const EdgeInsets.only(right: 10),
                child: icon,
              ),
              textWidget,
            ],
          );

    return SectionTile(
      key: key,
      dense: dense,
      // icon: icon,
      title: Center(
        child: title,
      ),
      color: color,
      onTap: onTap,
    );
  }

  static SectionTile dropdownButton<T>({
    Key? key,
    required String text,
    String? subTitle,
    Icon? icon,
    required Color color,
    List<T>? items,
    List<DropdownMenuItem<T>>? dropdownMenuItems,
    ValueSetter<T?>? onChanged,
    T? value,
    bool isDense = false,
    bool isExpanded = false,
    double? itemHeight = kMinInteractiveDimension,
    Color? focusColor,
    FocusNode? focusNode,
    bool autofocus = false,
    Color? dropdownColor,
    double? menuMaxHeight,
    bool dense = false,
  }) {
    dropdownMenuItems ??= items!
        .map((e) => DropdownMenuItem<T>(
              child: Text(e.toString()),
              value: e,
            ))
        .toList();

    return SectionTile(
      key: key,
      text: text,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: Container(
        child: DropdownButton<T>(
          items: dropdownMenuItems,
          value: value,
          onChanged: onChanged,
          isDense: isDense,
          isExpanded: isExpanded,
          itemHeight: itemHeight,
          focusColor: focusColor,
          focusNode: focusNode,
          autofocus: autofocus,
          dropdownColor: dropdownColor,
          menuMaxHeight: menuMaxHeight,
        ),
      ),
    );
  }

  static SectionTile popupMenuButton<T>({
    Key? key,
    required String text,
    String? subTitle,
    List<T>? items,
    Icon? icon,
    Icon? popupIcon,
    VoidCallback? onCanceled,
    required Color color,
    List<PopupMenuEntry<T>> Function(BuildContext)? itemBuilder,
    T? initialValue,
    ValueSetter<T>? onSelected,
    String? tooltip,
    double? elevation,
    EdgeInsetsGeometry padding = const EdgeInsets.all(8.0),
    Offset offset = Offset.zero,
    bool enabled = true,
    ShapeBorder? shape,
    Color? popUpColor,
    bool? enableFeedback,
    bool dense = false,
  }) {
    final GlobalKey _menuKey = new GlobalKey();

    popupIcon ??= Icon(Icons.arrow_drop_down);
    itemBuilder ??= (context) {
      return items!
          .map((e) => PopupMenuItem<T>(
                child: Text(e.toString()),
                value: e,
              ))
          .toList();
    };

    return SectionTile(
      key: key,
      text: text,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      onTap: () => (_menuKey.currentState as PopupMenuButtonState).showButtonMenu(),
      trailing: Container(
        child: PopupMenuButton<T>(
          key: _menuKey,
          icon: popupIcon,
          itemBuilder: itemBuilder,
          color: popUpColor,
          elevation: elevation,
          enableFeedback: enableFeedback,
          enabled: enabled,
          initialValue: initialValue,
          offset: offset,
          onCanceled: onCanceled,
          onSelected: onSelected,
          padding: padding,
          shape: shape,
          tooltip: tooltip,
        ),
      ),
    );
  }

  factory SectionTile.switchButton({
    Key? key,
    required String text,
    String? subTitle,
    Icon? icon,
    required ValueSetter<bool>? onChanged,
    required Color color,
    Color? activeColor,
    required bool switchButtonValue,
    bool dense = false,
  }) {
    return SectionTile(
      key: key,
      text: text,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: Switch(
        materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
        value: switchButtonValue,
        onChanged: onChanged,
        activeColor: activeColor,
      ),
      onTap: onChanged != null
          ? () {
              onChanged(!switchButtonValue);
            }
          : null,
    );
  }

  factory SectionTile.checkbox({
    Key? key,
    required String text,
    String? subTitle,
    Icon? icon,
    required ValueChanged<bool?>? onChanged,
    required Color color,
    Color? activeColor,
    required bool value,
    bool dense = false,
  }) {
    return SectionTile(
      key: key,
      text: text,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: Checkbox(
        value: value,
        onChanged: onChanged,
        activeColor: activeColor,
        checkColor: activeColor,
        materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
        tristate: false,
      ),
      onTap: onChanged != null
          ? () {
              onChanged(!value);
            }
          : null,
    );
  }

  factory SectionTile.scrollWheelTimeSelector({
    Key? key,
    required String text,
    String? subTitle,
    Icon? icon,
    required ValueChanged<TimeOfDay?> onChanged,
    required Color color,
    bool dense = false,
  }) {
    return SectionTile(
      key: key,
      text: text,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: Container(
        width: 110,
        child: ScrollWheelTimeSelector(
          color: color,
          height: 100,
          onTimeChanged: onChanged,
          controller: ScrollWheelTimeSelectorController(hasError: false),
        ),
      ),
      // onTap: onChanged != null
      //     ? () {
      //         onChanged(!value);
      //       }
      //     : null,
    );
  }

  static SectionTile scrollWheel<T>({
    Key? key,
    required String text,
    String? subTitle,
    Icon? icon,
    required List<T> items,
    Duration? notificationDelay,
    required ValueSetter<T> onChange,
    TextStyle? textStyle,
    required Color color,
    double itemExtent = 20,
    double magnification = 1.2,
    bool useMagnifier = true,
    double diameterRatio = RenderListWheelViewport.defaultDiameterRatio,
    required double width,
    double height = 60,
    bool dense = false,
  }) {
    return SectionTile(
      key: key,
      text: text,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: ScrollWheel<T>(
        color: color,
        height: height,
        items: items,
        onChange: onChange,
        diameterRatio: diameterRatio,
        itemExtent: itemExtent,
        magnification: magnification,
        notificationDelay: notificationDelay,
        textStyle: textStyle,
        useMagnifier: useMagnifier,
        width: width,
      ),
    );
  }

  static SectionTile scrollWheelInfinite<T>({
    Key? key,
    required String text,
    String? subTitle,
    Icon? icon,
    required List<T> items,
    Duration? notificationDelay,
    required ValueSetter<T> onChange,
    TextStyle? textStyle,
    required Color color,
    double itemExtent = 20,
    double magnification = 1.2,
    bool useMagnifier = true,
    double diameterRatio = RenderListWheelViewport.defaultDiameterRatio,
    required double width,
    double height = 60,
    bool dense = false,
  }) {
    return SectionTile(
      key: key,
      text: text,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: ScrollWheelInfinite<T>(
        color: color,
        height: height,
        items: items,
        onChange: onChange,
        diameterRatio: diameterRatio,
        itemExtent: itemExtent,
        magnification: magnification,
        notificationDelay: notificationDelay,
        textStyle: textStyle,
        useMagnifier: useMagnifier,
        width: width,
      ),
    );
  }
}

==> 1.0.2.tar.gz: lib/src/sections/SectionTile.dart <==
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

import '../../dotup_flutter_widgets.dart';

class SectionTile extends StatelessWidget {
  final String? titleText;
  final Widget? icon;
  final VoidCallback? onTap;
  final Color? color;
  final Widget? subTitle;
  final Widget? title;
  final Widget? trailing;
  final bool dense;

  const SectionTile({
    Key? key,
    this.titleText,
    this.icon,
    this.onTap,
    this.color,
    this.title,
    this.subTitle,
    this.trailing,
    this.dense = false,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ListTile(
      contentPadding: EdgeInsets.symmetric(horizontal: 10),
      dense: dense,
      minLeadingWidth: 10,
      leading: icon,
      title: title ??
          Text(
            titleText!,
            style: TextStyle(color: color),
          ),
      subtitle: subTitle,
      trailing: trailing,
      onTap: onTap,
      enabled: onTap != null,
    );
  }

  factory SectionTile.navigator({
    Key? key,
    required String titleText,
    Icon? icon,
    required VoidCallback onTap,
    required Color color,
    String? subTitle,
    bool dense = false,
  }) {
    return SectionTile(
      titleText: titleText,
      subTitle: subTitle == null ? null : Text(subTitle),
      color: color,
      dense: dense,
      trailing: Icon(
        Icons.keyboard_arrow_right,
        color: color,
      ),
      onTap: onTap,
    );
  }

  factory SectionTile.button({
    Key? key,
    required String titleText,
    Icon? icon,
    required VoidCallback onTap,
    required Color color,
    String? subTitle,
    bool dense = false,
  }) {
    final textWidget = Text(
      titleText,
      style: TextStyle(color: color),
    );

    final title = icon == null
        ? textWidget
        : Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Padding(
                padding: const EdgeInsets.only(right: 10),
                child: icon,
              ),
              textWidget,
            ],
          );

    return SectionTile(
      key: key,
      dense: dense,
      // icon: icon,
      title: Center(
        child: title,
      ),
      color: color,
      onTap: onTap,
    );
  }

  static SectionTile dropdownButton<T>({
    Key? key,
    required String titleText,
    String? subTitle,
    Icon? icon,
    required Color color,
    List<T>? items,
    List<DropdownMenuItem<T>>? dropdownMenuItems,
    ValueSetter<T?>? onChanged,
    T? value,
    bool isDense = false,
    bool isExpanded = false,
    double? itemHeight = kMinInteractiveDimension,
    Color? focusColor,
    FocusNode? focusNode,
    bool autofocus = false,
    Color? dropdownColor,
    double? menuMaxHeight,
    bool dense = false,
  }) {
    dropdownMenuItems ??= items!
        .map((e) => DropdownMenuItem<T>(
              child: Text(e.toString()),
              value: e,
            ))
        .toList();

    return SectionTile(
      key: key,
      titleText: titleText,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: Container(
        child: DropdownButton<T>(
          items: dropdownMenuItems,
          value: value,
          onChanged: onChanged,
          isDense: isDense,
          isExpanded: isExpanded,
          itemHeight: itemHeight,
          focusColor: focusColor,
          focusNode: focusNode,
          autofocus: autofocus,
          dropdownColor: dropdownColor,
          menuMaxHeight: menuMaxHeight,
        ),
      ),
    );
  }

  static SectionTile popupMenuButton<T>({
    Key? key,
    required String titleText,
    String? subTitle,
    List<T>? items,
    Icon? icon,
    Icon? popupIcon,
    VoidCallback? onCanceled,
    required Color color,
    List<PopupMenuEntry<T>> Function(BuildContext)? itemBuilder,
    T? initialValue,
    ValueSetter<T>? onSelected,
    String? tooltip,
    double? elevation,
    EdgeInsetsGeometry padding = const EdgeInsets.all(8.0),
    Offset offset = Offset.zero,
    bool enabled = true,
    ShapeBorder? shape,
    Color? popUpColor,
    bool? enableFeedback,
    bool dense = false,
  }) {
    final GlobalKey _menuKey = new GlobalKey();

    popupIcon ??= Icon(Icons.arrow_drop_down);
    itemBuilder ??= (context) {
      return items!
          .map((e) => PopupMenuItem<T>(
                child: Text(e.toString()),
                value: e,
              ))
          .toList();
    };

    return SectionTile(
      key: key,
      titleText: titleText,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      onTap: () => (_menuKey.currentState as PopupMenuButtonState).showButtonMenu(),
      trailing: Container(
        child: PopupMenuButton<T>(
          key: _menuKey,
          icon: popupIcon,
          itemBuilder: itemBuilder,
          color: popUpColor,
          elevation: elevation,
          enableFeedback: enableFeedback,
          enabled: enabled,
          initialValue: initialValue,
          offset: offset,
          onCanceled: onCanceled,
          onSelected: onSelected,
          padding: padding,
          shape: shape,
          tooltip: tooltip,
        ),
      ),
    );
  }

  factory SectionTile.switchButton({
    Key? key,
    required String titleText,
    String? subTitle,
    Icon? icon,
    required ValueSetter<bool>? onChanged,
    required Color color,
    Color? activeColor,
    required bool switchButtonValue,
    bool dense = false,
  }) {
    return SectionTile(
      key: key,
      titleText: titleText,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: Switch(
        materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
        value: switchButtonValue,
        onChanged: onChanged,
        activeColor: activeColor,
      ),
      onTap: onChanged != null
          ? () {
              onChanged(!switchButtonValue);
            }
          : null,
    );
  }

  factory SectionTile.checkbox({
    Key? key,
    required String titleText,
    String? subTitle,
    Icon? icon,
    required ValueChanged<bool?>? onChanged,
    required Color color,
    Color? activeColor,
    required bool value,
    bool dense = false,
  }) {
    return SectionTile(
      key: key,
      titleText: titleText,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: Checkbox(
        value: value,
        onChanged: onChanged,
        activeColor: activeColor,
        checkColor: activeColor,
        materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
        tristate: false,
      ),
      onTap: onChanged != null
          ? () {
              onChanged(!value);
            }
          : null,
    );
  }

  factory SectionTile.scrollWheelTimeSelector({
    Key? key,
    required String titleText,
    String? subTitle,
    Icon? icon,
    required ValueChanged<TimeOfDay?> onChanged,
    required Color color,
    bool dense = false,
  }) {
    return SectionTile(
      key: key,
      titleText: titleText,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: Container(
        width: 110,
        child: ScrollWheelTimeSelector(
          color: color,
          height: 100,
          onTimeChanged: onChanged,
          controller: ScrollWheelTimeSelectorController(hasError: false),
        ),
      ),
      // onTap: onChanged != null
      //     ? () {
      //         onChanged(!value);
      //       }
      //     : null,
    );
  }

  static SectionTile scrollWheel<T>({
    Key? key,
    required String titleText,
    String? subTitle,
    Icon? icon,
    required List<T> items,
    Duration? notificationDelay,
    required ValueSetter<T> onChange,
    TextStyle? textStyle,
    required Color color,
    double itemExtent = 20,
    double magnification = 1.2,
    bool useMagnifier = true,
    double diameterRatio = RenderListWheelViewport.defaultDiameterRatio,
    required double width,
    double height = 60,
    bool dense = false,
  }) {
    return SectionTile(
      key: key,
      titleText: titleText,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: ScrollWheel<T>(
        color: color,
        height: height,
        items: items,
        onChange: onChange,
        diameterRatio: diameterRatio,
        itemExtent: itemExtent,
        magnification: magnification,
        notificationDelay: notificationDelay,
        textStyle: textStyle,
        useMagnifier: useMagnifier,
        width: width,
      ),
    );
  }

  static SectionTile scrollWheelInfinite<T>({
    Key? key,
    required String titleText,
    String? subTitle,
    Icon? icon,
    required List<T> items,
    Duration? notificationDelay,
    required ValueSetter<T> onChange,
    TextStyle? textStyle,
    required Color color,
    double itemExtent = 20,
    double magnification = 1.2,
    bool useMagnifier = true,
    double diameterRatio = RenderListWheelViewport.defaultDiameterRatio,
    required double width,
    double height = 60,
    bool dense = false,
  }) {
    return SectionTile(
      key: key,
      titleText: titleText,
      subTitle: subTitle == null ? null : Text(subTitle),
      icon: icon,
      color: color,
      dense: dense,
      trailing: ScrollWheelInfinite<T>(
        color: color,
        height: height,
        items: items,
        onChange: onChange,
        diameterRatio: diameterRatio,
        itemExtent: itemExtent,
        magnification: magnification,
        notificationDelay: notificationDelay,
        textStyle: textStyle,
        useMagnifier: useMagnifier,
        width: width,
      ),
    );
  }
}
